@@ -1,6 +1,6 @@
 ;;; smtpmail.el --- simple SMTP protocol (RFC 821) for sending mail
 
-;; Copyright (C) 1995, 1996, 2001 Free Software Foundation, Inc.
+;; Copyright (C) 1995, 1996, 2001, 2002 Free Software Foundation, Inc.
 
 ;; Author: Tomoji Kagatani <kagatani@rbc.ncl.omron.co.jp>
 ;; Maintainer: Simon Josefsson <simon@josefsson.org>
@@ -13,22 +13,22 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
+;; XEmacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with XEmacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
-;;; Synched up with: FSF 21.1
+;;; Synched up with: Emacs 21.3 from CVS 2002-09-17
 
 ;;; Commentary:
 
@@ -42,12 +42,12 @@
 ;;(setq smtpmail-local-domain "YOUR DOMAIN NAME")
 ;;(setq smtpmail-sendto-domain "YOUR DOMAIN NAME")
 ;;(setq smtpmail-debug-info t) ; only to debug problems
-;;(setq smtpmail-auth-credentials
+;;(setq smtpmail-auth-credentials  ; or use ~/.authinfo
 ;;      '(("YOUR SMTP HOST" 25 "username" "password")))
 ;;(setq smtpmail-starttls-credentials
 ;;      '(("YOUR SMTP HOST" 25 "~/.my_smtp_tls.key" "~/.my_smtp_tls.cert")))
 
-;; To queue mail, set smtpmail-queue-mail to t and use 
+;; To queue mail, set smtpmail-queue-mail to t and use
 ;; smtpmail-send-queued-mail to send.
 
 ;; Modified by Stephen Cranefield <scranefield@infoscience.otago.ac.nz>,
@@ -68,7 +68,6 @@
 ;;; Code:
 
 (require 'sendmail)
-(require 'time-stamp)
 (autoload 'starttls-open-stream "starttls")
 (autoload 'starttls-negotiate "starttls")
 (autoload 'mail-strip-quoted-names "mail-utils")
@@ -76,7 +75,10 @@
 (autoload 'message-make-date "message")
 (autoload 'message-make-message-id "message")
 (autoload 'rfc2104-hash "rfc2104")
-(require 'base64)
+(autoload 'base64-decode-string "base64")
+(autoload 'netrc-parse "netrc")
+(autoload 'netrc-machine "netrc")
+(autoload 'netrc-get "netrc")
 
 ;;;
 (defgroup smtpmail nil
@@ -85,11 +87,12 @@
 
 
 (defcustom smtpmail-default-smtp-server nil
-  "*Specify default SMTP server."
+  "*Specify default SMTP server.
+This only has effect if you specify it before loading the smtpmail library."
   :type '(choice (const nil) string)
   :group 'smtpmail)
 
-(defcustom smtpmail-smtp-server 
+(defcustom smtpmail-smtp-server
   (or (getenv "SMTPSERVER") smtpmail-default-smtp-server)
   "*The name of the host running SMTP server."
   :type '(choice (const nil) string)
@@ -142,7 +145,7 @@ The commands enables verbose information from the SMTP server."
   :type 'boolean
   :group 'smtpmail)
 
-(defcustom smtpmail-queue-mail nil 
+(defcustom smtpmail-queue-mail nil
   "*Specify if mail is queued (if t) or sent immediately (if nil).
 If queued, it is stored in the directory `smtpmail-queue-dir'
 and sent with `smtpmail-send-queued-mail'."
@@ -154,18 +157,20 @@ and sent with `smtpmail-send-queued-mail'."
   :type 'directory
   :group 'smtpmail)
 
-(defcustom smtpmail-auth-credentials '(("" 25 "" nil))
-  "Specify username and password for servers.
-It is a list of four-element lists that contain, in order,
+(defcustom smtpmail-auth-credentials "~/.authinfo"
+  "Specify username and password for servers, directly or via .netrc file.
+This variable can either be a filename pointing to a file in netrc(5)
+format, or list of four-element lists that contain, in order,
 `servername' (a string), `port' (an integer), `user' (a string) and
-`password' (a string, or nil to query the user when needed).
-If you need to enter a `realm' too, add it to the user string, so that
-it looks like `user@realm'."
-  :type '(repeat (list (string  :tag "Server")
-		       (integer :tag "Port")
-		       (string  :tag "Username")
-		       (choice (const :tag "Query when needed" nil)
-			       (string  :tag "Password"))))
+`password' (a string, or nil to query the user when needed).  If you
+need to enter a `realm' too, add it to the user string, so that it
+looks like `user@realm'."
+  :type '(choice file
+		 (repeat (list (string  :tag "Server")
+			       (integer :tag "Port")
+			       (string  :tag "Username")
+			       (choice (const :tag "Query when needed" nil)
+				       (string  :tag "Password")))))
   :group 'smtpmail)
 
 (defcustom smtpmail-starttls-credentials '(("" 25 "" ""))
@@ -192,6 +197,8 @@ This is relative to `smtpmail-queue-dir'.")
 (defvar smtpmail-address-buffer)
 (defvar smtpmail-recipient-address-list)
 
+(defvar smtpmail-queue-counter 0)
+
 ;; Buffer-local variable.
 (defvar smtpmail-read-point)
 
@@ -205,6 +212,9 @@ This is relative to `smtpmail-queue-dir'.")
 ;;;
 ;;;
 
+(defvar smtpmail-mail-address nil
+  "Value of `user-mail-address' in ambient buffer.")
+
 ;;;###autoload
 (defun smtpmail-send-it ()
   (let ((errbuf (if mail-interactive
@@ -214,6 +224,7 @@ This is relative to `smtpmail-queue-dir'.")
 	(case-fold-search nil)
 	delimline
 	(mailbuf (current-buffer))
+	(smtpmail-mail-address (user-mail-address)) ;; XEmacs
 	;; XEmacs: Don't frob `smtpmail-code-conv-from' here
 	)
     (unwind-protect
@@ -252,7 +263,7 @@ This is relative to `smtpmail-queue-dir'.")
 	    ;; they put one in themselves.
 	    (goto-char (point-min))
 	    (if (not (re-search-forward "^From:" delimline t))
-		(let* ((login (user-mail-address)) ;; XEmacs
+		(let* ((login smtpmail-mail-address)
 		       (fullname (user-full-name)))
 		  (cond ((eq mail-from-style 'angles)
 			 (insert "From: " fullname)
@@ -286,7 +297,7 @@ This is relative to `smtpmail-queue-dir'.")
 			     ;; ... then undo escaping of matching parentheses,
 			     ;; including matching nested parentheses.
 			     (goto-char fullname-start)
-			     (while (re-search-forward 
+			     (while (re-search-forward
 				     "\\(\\=\\|[^\\]\\(\\\\\\\\\\)*\\)\\\\(\\(\\([^\\]\\|\\\\\\\\\\)*\\)\\\\)"
 				     fullname-end 1)
 			       (replace-match "\\1(\\3)" t)
@@ -321,7 +332,7 @@ This is relative to `smtpmail-queue-dir'.")
 	  (setq smtpmail-recipient-address-list
 		    (smtpmail-deduce-address-list tembuf (point-min) delimline))
 	  (kill-buffer smtpmail-address-buffer)
-	  
+
 	  ;; XEmacs change
 	  (mail-check-safe-charset)
 
@@ -329,16 +340,19 @@ This is relative to `smtpmail-queue-dir'.")
 	  ; Send or queue
 	  (if (not smtpmail-queue-mail)
 	      (if (not (null smtpmail-recipient-address-list))
-		  (if (not (smtpmail-via-smtp 
+		  (if (not (smtpmail-via-smtp
 			    smtpmail-recipient-address-list tembuf))
 		      (error "Sending failed; SMTP protocol error"))
 		(error "Sending failed; no recipients"))
-	    (let* ((file-data (concat 
-			       smtpmail-queue-dir
-			       (concat (time-stamp-yyyy-mm-dd)
-				       "_" (time-stamp-hh:mm:ss))))
-		      (file-data (convert-standard-filename file-data))
-		      (file-elisp (concat file-data ".el"))
+	    (let* ((file-data
+		    (expand-file-name
+		     (format "%s_%i"
+			     (format-time-string "%Y-%m-%d_%H:%M:%S")
+			     (setq smtpmail-queue-counter
+				   (1+ smtpmail-queue-counter)))
+		     smtpmail-queue-dir))
+		   (file-data (convert-standard-filename file-data))
+		   (file-elisp (concat file-data ".el"))
 		   (buffer-data (create-file-buffer file-data))
 		   (buffer-elisp (create-file-buffer file-elisp))
 		   (buffer-scratch "*queue-mail*"))
@@ -351,12 +365,12 @@ This is relative to `smtpmail-queue-dir'.")
 		(insert (concat
 			 "(setq smtpmail-recipient-address-list '"
 			 (prin1-to-string smtpmail-recipient-address-list)
-			 ")\n"))	    	    
+			 ")\n"))
 		(write-file file-elisp)
 		(set-buffer (generate-new-buffer buffer-scratch))
 		(insert (concat file-data "\n"))
-		(append-to-file (point-min) 
-				(point-max) 
+		(append-to-file (point-min)
+				(point-max)
 				smtpmail-queue-index)
 		)
 	      (kill-buffer buffer-scratch)
@@ -382,7 +396,7 @@ This is relative to `smtpmail-queue-dir'.")
 	;; the MIME headers, and code conversions might guess the
 	;; encoding wrongly.
 	(with-temp-buffer
-	  (let ((coding-system-for-read 'binary))
+	  (let ((coding-system-for-read 'binary)) ;; XEmacs
 	    (insert-file-contents file-msg))
 	  (if (not (null smtpmail-recipient-address-list))
 	      (if (not (smtpmail-via-smtp smtpmail-recipient-address-list
@@ -462,7 +476,17 @@ This is relative to `smtpmail-queue-dir'.")
 (defun smtpmail-try-auth-methods (process supported-extensions host port)
   (let* ((mechs (cdr-safe (assoc 'auth supported-extensions)))
 	 (mech (car (smtpmail-intersection smtpmail-auth-supported mechs)))
-	 (cred (smtpmail-find-credentials smtpmail-auth-credentials host port))
+	 (cred (if (stringp smtpmail-auth-credentials)
+		   (let* ((netrc (netrc-parse smtpmail-auth-credentials))
+			  (hostentry (netrc-machine
+				      netrc host (format "%s" (or port "smtp"))
+				      "smtp")))
+                     (when hostentry
+                       (list host port
+                             (netrc-get hostentry "login")
+                             (netrc-get hostentry "password"))))
+		 (smtpmail-find-credentials
+		  smtpmail-auth-credentials host port)))
 	 (passwd (when cred
 		   (or (smtpmail-cred-passwd cred)
 		       (read-passwd
@@ -509,7 +533,8 @@ This is relative to `smtpmail-queue-dir'.")
        (t
 	(error "Mechanism %s not implemented" mech)))
       ;; Remember the password.
-      (unless (smtpmail-cred-passwd cred)
+      (when (and (not (stringp smtpmail-auth-credentials))
+		 (null (smtpmail-cred-passwd cred)))
 	(setcar (cdr (cdr (cdr cred))) passwd)))))
 
 (defun smtpmail-via-smtp (recipient smtpmail-text-buffer)
@@ -545,7 +570,7 @@ This is relative to `smtpmail-queue-dir'.")
 	    (make-local-variable 'smtpmail-read-point)
 	    (setq smtpmail-read-point (point-min))
 
-	    
+
 	    (if (or (null (car (setq greeting (smtpmail-read-response process))))
 		    (not (integerp (car greeting)))
 		    (>= (car greeting) 400))
@@ -592,7 +617,7 @@ This is relative to `smtpmail-queue-dir'.")
 	    (if (and do-starttls
 		     (smtpmail-find-credentials smtpmail-starttls-credentials host port)
 		     (member 'starttls supported-extensions)
-		     (process-id process))
+		     (numberp (process-id process)))
 		(progn
 		  (smtpmail-send-command process (format "STARTTLS"))
 		  (if (or (null (car (setq response-code (smtpmail-read-response process))))
@@ -602,7 +627,7 @@ This is relative to `smtpmail-queue-dir'.")
 		  (starttls-negotiate process)
 		  (setq do-starttls nil))
 	      (setq do-ehlo nil))))
-	    
+
 	    (smtpmail-try-auth-methods process supported-extensions host port)
 
 	    (if (or (member 'onex supported-extensions)
@@ -669,8 +694,7 @@ This is relative to `smtpmail-queue-dir'.")
 ;	      (smtpmail-send-command process (format "MAIL FROM:%s@%s" (user-login-name) (smtpmail-fqdn)))
 	      (smtpmail-send-command process (format "MAIL FROM: <%s>%s%s"
 						     (or mail-envelope-from
-							 ;; XEmacs:
-							 (user-mail-address))
+							 smtpmail-mail-address)
 						     size-part
 						     body-part))
 
@@ -679,7 +703,7 @@ This is relative to `smtpmail-queue-dir'.")
 		      (>= (car response-code) 400))
 		  (throw 'done nil)
 		))
-	    
+
 	    ;; RCPT TO: <recipient>
 	    (let ((n 0))
 	      (while (not (null (nth n recipient)))
@@ -693,7 +717,7 @@ This is relative to `smtpmail-queue-dir'.")
 		    (throw 'done nil)
 		  )
 		))
-	    
+
 	    ;; DATA
 	    (smtpmail-send-command process "DATA")
 
@@ -730,7 +754,9 @@ This is relative to `smtpmail-queue-dir'.")
 ;		    (>= (car response-code) 400))
 ;		(throw 'done nil)
 ;	      )
-	    (delete-process process))))))
+	    (delete-process process)
+	    (unless smtpmail-debug-info
+	      (kill-buffer process-buffer)))))))
 
 
 (defun smtpmail-process-filter (process output)
@@ -755,7 +781,7 @@ This is relative to `smtpmail-queue-dir'.")
       (setq response-strings
 	    (cons (buffer-substring smtpmail-read-point (- match-end 2))
 		  response-strings))
-	
+
       (goto-char smtpmail-read-point)
       (if (looking-at "[0-9]+ ")
 	  (let ((begin (match-beginning 0))
@@ -770,10 +796,10 @@ This is relative to `smtpmail-queue-dir'.")
 		nil
 	      (setq response-continue nil)
 	      (setq return-value
-		    (cons (string-to-int 
-			   (buffer-substring begin end)) 
+		    (cons (string-to-int
+			   (buffer-substring begin end))
 			  (nreverse response-strings)))))
-	
+
 	(if (looking-at "[0-9]+-")
 	    (progn (if smtpmail-debug-info
 		     (message "%s" (car response-strings)))
@@ -782,7 +808,7 @@ This is relative to `smtpmail-queue-dir'.")
 	  (progn
 	    (setq smtpmail-read-point match-end)
 	    (setq response-continue nil)
-	    (setq return-value 
+	    (setq return-value
 		  (cons nil (nreverse response-strings)))
 	    )
 	  )))
@@ -843,7 +869,7 @@ This is relative to `smtpmail-queue-dir'.")
       )
     )
   )
-    
+
 
 (defun smtpmail-deduce-address-list (smtpmail-text-buffer header-start header-end)
   "Get address list suitable for smtp RCPT TO: <address>."
